"""
Usage: log-uploader [options] LOG_FILE LOG_GROUP LOG_STREAM
       log-uploader --version


This will upload the given log to stream, each line as a log event. Every
line in the log should start with an '[<ISO8601-TIMESTAMP>]' to allow this
script to correctly add that data to logged data.

This script assumes we can write data faster towards AWS than it is being
generated by the serial logger to the log file(s).

Options:
  --watch              Stay on foreground and upload new events.
  --ignore-timestamp   Ignore stored timestamp and upload all events.
  --verbose            Enable verbose logging.
"""

import boto3
import botocore
import docopt
import logging
import os
import time
import datetime
import re
import itertools
from typing import Optional, NamedTuple, Iterable
import multiprocessing as mp
import queue
from pathlib import Path

TIMESTAMP_FILE_SUFFIX = ".lus"
AWS_MAX_BATCH_SIZE = 1048576
AWS_MAX_EVENT_TIME_SPAN = 24 * 3600

logger = logging.getLogger(__name__)

here = Path(__file__).parent.resolve()


class ChunkUploader:
    def __init__(self,
                 client,
                 group: str,
                 stream: str,
                 upload_sequence_token: str,
                 min_time_between_requests: float,
                 max_lines_in_batch: int,
                 max_time_between_uploads: float,
                 max_batch_size: int = AWS_MAX_BATCH_SIZE,
                 max_event_time_span: int = AWS_MAX_EVENT_TIME_SPAN,
                 timestamp_file_name: Optional[str] = None):
        self.batch = []
        self.min_time_between_requests = min_time_between_requests
        self.max_time_between_uploads = max_time_between_uploads
        self.max_event_time_span = max_event_time_span
        self.max_batch_size = max_batch_size
        self.prev_ts = time.time()
        self.client = client
        self.group = group
        self.stream = stream

        self.timestamp_file_name = timestamp_file_name
        self.upload_sequence_token = upload_sequence_token
        self._last_event_ts = None
        self._current_batch_size = 0
        self.max_lines_in_batch = max_lines_in_batch

    def append(self, event):
        # Timestamps need to be increasing in a batch.
        if self._last_event_ts and self._last_event_ts > event.timestamp:
            self.upload_current_batch()

        if self.batch and (event.timestamp - self.batch[0].timestamp) > datetime.timedelta(seconds=self.max_event_time_span):
            self.upload_current_batch()

        EVENT_HEADER_SIZE = 26
        event_size = len(event.message.encode('utf-8')) + EVENT_HEADER_SIZE
        if event_size + self._current_batch_size > self.max_batch_size:
            self.upload_current_batch()

        if event.message:
            self.batch.append(event)
            self._current_batch_size += event_size
        self._last_event_ts = event.timestamp

        self.maybe_upload()

    def maybe_upload(self):
        if len(self.batch) >= self.max_lines_in_batch:
            self.upload_current_batch()
        else:
            elapsed = self._elapsed()
            if elapsed and elapsed > self.max_time_between_uploads:
                self.upload_current_batch()

    def _update_timestamp(self, ts: datetime.datetime):
        if not self.timestamp_file_name:
            return

        try:
            with open(self.timestamp_file_name, "w") as fp:
                fp.write(ts.isoformat())
        except PermissionError:
            logger.debug("Can't write timestamp file")

    def upload_current_batch(self):
        if not self.batch:
            return

        self._maybe_throttle()
        logger.debug(f"Uploading batch")

        # From https://docs.aws.amazon.com/AmazonCloudWatchLogs/latest/APIReference/API_PutLogEvents.html
        #  The batch of events must satisfy the following constraints:
        #
        # The maximum batch size is 1,048,576 bytes. This size is
        # calculated as the sum of all event messages in UTF-8, plus 26
        # bytes for each log event.
        # None of the log events in the batch can be more than 2 hours in
        # the future.
        # None of the log events in the batch can be older than 14 days or
        # older than the retention period of the log group.
        # The log events in the batch must be in chronological order by
        # their timestamp. The timestamp is the time the event occurred,
        # expressed as the number of milliseconds after Jan 1,
        # 1970 00:00:00 UTC. (In AWS Tools for PowerShell and the AWS SDK
        # for .NET, the timestamp is specified in .NET format:
        # yyyy-mm-ddThh:mm:ss. For example, 2017-09-15T13:45:30.)
        # A batch of log events in a single request cannot span more than
        # 24 hours. Otherwise, the operation fails.
        # The maximum number of log events in a batch is 10,000.
        # There is a quota of 5 requests per second per log stream.
        # Additional requests are throttled. This quota can't be changed.

        self.upload_sequence_token = upload_batch(self.client, self.group,
                                                  self.stream, self.batch,
                                                  self.upload_sequence_token)
        self._update_timestamp(self._last_event_ts)
        self.batch = []
        self.prev_ts = time.time()
        self._current_batch_size = 0

    def _maybe_throttle(self):
        if self.prev_ts is not None:
            elapsed = self._elapsed()
            if elapsed < self.min_time_between_requests:
                throttle_seconds = self.min_time_between_requests - elapsed
                logger.info(
                    f"Throttling for {throttle_seconds}s to avoid hitting "
                    f"request quota")
                time.sleep(throttle_seconds)

    def _elapsed(self):
        if not self.prev_ts:
            return None
        else:
            elapsed = time.time() - self.prev_ts
            return elapsed


class EOF:
    pass


def log_entry_producer(q, log_filename, watch: bool = False, read_timestamp: bool = True):
    try:
        for ev in get_log_entries(log_filename, watch=watch, read_timestamp=read_timestamp):
            q.put(ev)
        q.put(EOF())
    except Exception as e:
        q.put(e)
        raise


def upload_log(name: str, group: str, stream: str, watch: bool = False,
               max_time_between_uploads: float = 5,
               min_time_between_requests: float = 0.2,
               max_lines_in_batch: int = 5000,
               max_batch_size: int = AWS_MAX_BATCH_SIZE,
               timestamp_file_name: Optional[str] = None,
               read_timestamp: bool = True):
    """

    :param name: Name of the log file.
    :param group: Log Group in AWS CloudWatch.
    :param stream: Log Stream in AWS CloudWatch.
    :param watch: Instead of returning when log file is at the end, keep reading and waiting for more input.
    :param max_time_between_uploads: If there is any input, avoid waiting longer than this to upload.
    :param min_time_between_requests: Wait at least this long between calls to avoid hitting request quota.
    :param max_lines_in_batch: Maximum number of events in a single batch.
    :param max_batch_size: Maximum size, in bytes, for a single batch.
    :param timestamp_file_name: Timestamp file name, leave as None for default.
    :param read_timestamp: If True, skip events while events have earlier time than the timestamp found in the `timestamp_file_name`.
    :return:
    """
    client = boto3.client('logs')

    upload_sequence_token = get_next_sequence_token(client, group, stream)

    if timestamp_file_name is None:
        timestamp_file_name = get_stamp_filename(name)
    uploader = ChunkUploader(client, group, stream, upload_sequence_token,
                             min_time_between_requests,
                             max_lines_in_batch,
                             max_time_between_uploads,
                             max_batch_size,
                             timestamp_file_name=timestamp_file_name)

    q = mp.Queue()
    producer = mp.Process(target=log_entry_producer, args=(q, name),
                          kwargs={'watch': watch, 'read_timestamp': read_timestamp})
    producer.daemon = True
    producer.start()

    while True:
        try:
            ev = q.get(block=True, timeout=max(max_time_between_uploads/5, 0.1))
            if isinstance(ev, EOF):
                logger.info("Got EOF")
                break
            elif isinstance(ev, Exception):
                logger.error(f"Reader encountered error; {ev}", exc_info=ev)
                break
            else:
                logger.debug(f"got line {ev.message}")
                uploader.append(ev)
        except queue.Empty:
            uploader.maybe_upload()
            if not producer.is_alive():
                logger.error("Producer seems to have died.")
                break

    logger.info("stopping")
    uploader.upload_current_batch()
    q.close()
    producer.join()


def upload_batch(client, group, stream, batch, token):
    rest = {}
    if token:
        rest['sequenceToken'] = token
    response = client.put_log_events(
        logGroupName=group,
        logStreamName=stream,
        logEvents=[dict(timestamp=int(ev.timestamp.timestamp() * 1000),
                        message=ev.message) for ev in batch],
        **rest)
    logger.info(f"Wrote {len(batch)} log events, rejections: "
                f"{response.get('rejectedLogEventsInfo', '<none>')}")
    return response.get('nextSequenceToken', "")


def get_next_sequence_token(client, group, stream):
    response = client.describe_log_streams(logGroupName=group,
                                           logStreamNamePrefix=stream)
    matching = list(filter(lambda xx: xx['logStreamName'] == stream, response['logStreams']))
    if matching:
        stream_description = matching[0]
        logger.debug(f"Stream description for {stream}: {stream_description}")
        sequence_token = stream_description.get('uploadSequenceToken', "")
    else:
        try:
            client.create_log_stream(logGroupName=group, logStreamName=stream)
        except botocore.exceptions.ClientError as e:
            logger.warning(f"Failed to create stream {stream} to log group {group}; {e}")
        sequence_token = ""
    return sequence_token


class Event(NamedTuple):
    timestamp: datetime.datetime
    message: str


def parse_log_line(line: Optional[str]) -> Optional[Event]:
    if line is None:
        return None

    m = re.search(r"(?P<timestamp>\d+-\d+-\d+[T ]\d{2}:\d{2}:\d{2}(\.\d+)?([+-]\d{2}:\d{2})?)]?:? ?(?P<msg>.*$)", line)
    if m is None:
        logger.warning(f"line \"{line}\" did not match when finding timestamp")
        return None

    timestamp = m.group("timestamp")
    try:
        dt = datetime.datetime.fromisoformat(timestamp)
    except ValueError:
        logger.warning(f"line \"{line}\" had an invalid timestamp.")
        return None

    if dt.tzinfo is None:
        # Treat as old format localtime log timestamp
        dt = dt.astimezone()
    msg = m.group("msg")
    return Event(timestamp=dt, message=msg)


def get_log_entries(name: str, watch: bool = False, read_timestamp: bool = True) -> Iterable[Event]:
    """
    Return a generator yielding log events. If `watch` is True, tracks file
    changes to continue reading from a newly created file with the same name.

    :param name:
    :param watch:
    :param read_timestamp:
    :return:
    """
    if read_timestamp:
        timestamp = get_timestamp(name)
    else:
        timestamp = None

    def get_lines(fp):
        line = "start"
        while line or watch:
            line = fp.readline()
            if line:
                yield line
            else:
                if watch:
                    # in case file is just being rotated.
                    if os.path.exists(name) and os.stat(name).st_ino != current_inode:
                        return
                    else:
                        time.sleep(0.5)
                        continue
                else:
                    return None

    while True:
        try:
            current_inode = os.stat(name).st_ino
            with open(name, errors="backslashreplace") as logfile:
                for event in itertools.dropwhile(
                        lambda arg: timestamp is not None and arg and arg.timestamp <= timestamp,
                        filter(None, (parse_log_line(line) for line in get_lines(logfile)))):
                    yield event
        except OSError as e:
            if e.errno == 2:
                time.sleep(0.5)
            else:
                raise

        if not watch:
            return


def get_timestamp(name) -> Optional[datetime.datetime]:
    state_file_name = get_stamp_filename(name)
    if os.path.exists(state_file_name):
        line = parse_log_line(open(state_file_name).read().strip())
        if line:
            return line.timestamp
        else:
            return None
    else:
        return None


def get_stamp_filename(name):
    return name + TIMESTAMP_FILE_SUFFIX


def output_version():
    print("Metsuri log collector, version: " + open(here / "VERSION").read().strip())


def main():
    mp.set_start_method('spawn')

    opts = docopt.docopt(__doc__)

    if opts['--version']:
        output_version()
        return

    if opts['--verbose']:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(level=level,
                        format='%(asctime)s [%(levelname)s] %('
                               'filename)s:%(lineno)s %(funcName)s %('
                               'message)s',
                        datefmt="%Y-%m-%dT%H:%M:%S%z"
                        )

    upload_log(opts["LOG_FILE"], opts["LOG_GROUP"], opts["LOG_STREAM"],
               watch=opts['--watch'], read_timestamp=not opts['--ignore-timestamp'])
    logger.info("exiting")


if __name__ == "__main__":
    main()



"""
Usage: log-check [options] LOG_FILE

Options:
  --verbose            Enable verbose logging.

"""
import docopt
import logging
from metsuri.log_uploader import get_log_entries

logger = logging.getLogger(__name__)


def main():
    opts = docopt.docopt(__doc__)
    if opts['--verbose']:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(level=level,
                        format='%(asctime)s [%(levelname)s] %('
                               'filename)s:%(lineno)s %(funcName)s %('
                               'message)s',
                        datefmt="%Y-%m-%dT%H:%M:%S%z"
                        )
    try:
        line_number = 0
        for line_number, line in enumerate(get_log_entries(opts["LOG_FILE"])):
            pass
        else:
            print(f"Successfully processed {line_number} lines of log.")
    except Exception as e:
        logger.exception(f"Got error {e}")


if __name__ == "__main__":
    main()


"""
Usage: serial-logger [options] PORT LOGFILE

Options:
  --no-stdout          Disable echoing to stdout.
  --rotate-every INTERVAL
                       Rotate logs every INTERVAL time period. Specify as
                       days (d), hours (h), minutes (m) and seconds (s).
                       For example, 1d12h30s. [default: 1d]
  --retain NUM         Retain this many rotated logs. [default: 5]
  --verbose            Enable verbose logging (internal to the logger).
"""
import serial
import docopt
import os
import datetime
import logging
from typing import Optional
from metsuri.log_download import parse_interval
from metsuri.log_uploader import parse_log_line
import time

logger = logging.getLogger(__name__)


class LogFile:
    def __init__(self,
                 filename: str,
                 rotation_interval: Optional[datetime.timedelta] = None,
                 num_retained_logfiles: Optional[int] = None,
                 copy_to_stdout: bool = False):
        self.filename = filename
        self.rotation_interval = rotation_interval
        self.num_retained_logfiles = num_retained_logfiles
        self.file = open(filename, "a+")

        self.rotation_timestamp = self._get_rotation_timestamp()
        self.to_stdout = copy_to_stdout
        self._write_line_internal("**** started logging ****")

    def _get_rotation_timestamp(self) -> datetime.datetime:
        """
        Obtain accurate timestamp of the last rotation from the beginning of
        an existing log file.

        :return: ts
        """
        cur_pos = self.file.tell()
        if cur_pos > 0:
            self.file.seek(0)
            line = self.file.readline()
            ts, _ = parse_log_line(line)
            self.file.seek(cur_pos)
        else:
            ts = datetime.datetime.now(datetime.timezone.utc)
        return ts

    def __enter__(self):
        return self

    def __exit__(self, *exc):
        self._write_line_internal("**** stopped logging ****")
        self.file.close()

    @staticmethod
    def _parse_int(string):
        try:
            return int(string)
        except ValueError:
            return None

    def rotate_logs(self):
        self.file.close()
        log_dir, base = os.path.split(self.filename)
        log_dir = log_dir if log_dir else "."
        to_rotate = filter(
            None,
            map(lambda fn: self._parse_int(fn[len(base + "."):]),
                filter(lambda fn: fn.startswith(base + "."),
                       os.listdir(log_dir))))

        for suffix_int in sorted(to_rotate, reverse=True):
            filename = f"{self.filename}.{suffix_int}"
            if suffix_int >= self.num_retained_logfiles:
                os.remove(filename)
            else:
                os.rename(filename, f"{self.filename}.{suffix_int + 1}")

        os.rename(self.filename, f"{self.filename}.1")
        self.rotation_timestamp = datetime.datetime.now(datetime.timezone.utc)
        self.file = open(self.filename, "w")

    def _write_line_internal(self, line):
        formatted = self.format_line(line)
        if self.to_stdout:
            print(formatted, end='')
        self.file.write(formatted)
        self.file.flush()
        return formatted

    def write_line(self, line):
        if self.rotation_interval is not None:
            if datetime.datetime.now(datetime.timezone.utc) - self.rotation_timestamp >= self.rotation_interval:
                self.rotate_logs()
        return self._write_line_internal(line)

    @classmethod
    def timestamp(cls):
        return datetime.datetime.now(datetime.timezone.utc).isoformat(timespec='milliseconds')

    @classmethod
    def format_line(cls, line):
        return f"{cls.timestamp()} {line.rstrip()}\n"


def collect_serial_debug(port, logfile, disable_stdout=False):
    disconnected = True
    try:
        while True:
            try:
                if disconnected:
                    ser = serial.Serial(port=port, baudrate=115200,
                                        timeout=None)
                    logfile.write_line("**** USB connected ****")
                    disconnected = False
                line = ser.readline()
            except serial.SerialException as e:
                # most likely USB has been disconnected. wait a bit and retry.
                if not disconnected:
                    try:
                        # Try to avoid case where the USB device changes path
                        ser.close()
                    except serial.SerialException as extra:
                        pass
                    logfile.write_line("**** USB disconnected ****")
                    disconnected = True
                    logger.error("USB disconnected with error", exc_info=e)
                else:
                    logger.info("Got error, USB already disconnected", exc_info=e)
                time.sleep(1)
                continue
            logfile.write_line(line.decode('ascii', errors='replace'))
    except StopIteration:
        pass


def main():
    opts = docopt.docopt(__doc__)
    if opts['--verbose']:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(level=level,
                        format='%(asctime)s [%(levelname)s] %('
                               'filename)s:%(lineno)s %(funcName)s %('
                               'message)s',
                        datefmt="%Y-%m-%dT%H:%M:%S%z"
                        )
    with LogFile(opts['LOGFILE'],
                 rotation_interval=parse_interval(opts['--rotate-every']),
                 num_retained_logfiles=int(opts['--retain']),
                 copy_to_stdout=not bool(opts['--no-stdout'])) as logfile:
        collect_serial_debug(opts['PORT'], logfile, opts['--no-stdout'])


if __name__ == '__main__':
    main()



"""
Usage: log-download [options] LOG_GROUP STREAM_PREFIX OUTPUT
       log-download list-streams [options] LOG_GROUP [STREAM_PREFIX]

Export log streams with prefix STREAM_PREFIX from LOG_GROUP.

Command list-streams can be used to enumerate possible log-streams.

The logs are downloaded to the OUTPUT directory, which is created if it
does not exist.

Time period handling:

The `--from`, `--to` and `--interval` specify the period from which the
logs are dumped. If both `--from` and `--to` are specified, those will be
used and `--interval` will be ignored. If either `--to` or `--from` is
specified along with `--interval`, the period will be calculated from the
values. If only `--from` is specified, logs are dumped from that time to
current time. If only `--to` is specified, a default `--interval` of "1d"
will be used.

If neither --from or --to is specified, a default value for --to is taken to
be the current time and result is calculated as above depending on --interval.

Options:
  --interval INTERVAL  Use INTERVAL as the time window.
  --from TIME          Use TIME as the beginning of the window, specified in
                       ISO-8601 format.
  --to TIME            Use TIME as the end of the window, specified in
                       ISO-8601 format. If unspecified, use current time.
  --verbose            Enable verbose logging.
"""
import boto3
import logging
import docopt
import datetime
import os
import re
from rich.logging import RichHandler
import rich.progress as rp


logger = logging.getLogger(__name__)


def get_streams(client, log_group: str, stream_prefix: str):
    params = {}
    if stream_prefix:
        params['logStreamNamePrefix'] = stream_prefix
    # max 50 stream at this time
    response = client.describe_log_streams(
        logGroupName=log_group,
        **params
    )
    streams = [stream['logStreamName'] for stream in response['logStreams']]
    return streams


def log_download(client, log_group: str, stream_prefix: str, output: str,
                 from_time: datetime.datetime, to_time: datetime.datetime,
                 progress):

    streams = get_streams(client, log_group, stream_prefix)
    if not streams:
        print(f"No streams found from log group \"{log_group}\" with "
              f"prefix \"{stream_prefix}\"")
        return

    progress.console.log("Downloading " + ', '.join(streams))

    event_params = {'startTime': int(from_time.timestamp() * 1000),
                    'endTime': int(to_time.timestamp() * 1000)}

    start = event_params['startTime']
    total = event_params['endTime'] - start

    # If stream is empty, file should not be created
    for stream in progress.track(streams, description="All streams"):
        current_token = ""
        next_token = None

        current_params = event_params.copy()
        task = progress.add_task(f" {stream}", total=total)
        output_file = None
        while current_token != next_token:
            if next_token:
                current_params['nextToken'] = next_token
            response = client.get_log_events(
                logGroupName=log_group,
                logStreamName=stream,
                startFromHead=True,
                **current_params)
            current_token = next_token

            for ev in response['events']:
                if output_file is None:
                    os.makedirs(output, exist_ok=True)
                    output_file = open(os.path.join(output, stream), "w",
                                       encoding="utf-8")
                timestamp = datetime.datetime.fromtimestamp(
                    ev['timestamp'] / 1000,
                    tz=datetime.timezone.utc).astimezone().isoformat(
                    sep=' ',
                    timespec='milliseconds')
                print(f"{timestamp} {ev['message']}", file=output_file)
                progress.update(task, completed=ev['timestamp'] - start)
            next_token = response['nextForwardToken']
        progress.update(task, completed=total)


def parse_period(from_, given_interval, to):
    if given_interval:
        interval = parse_interval(given_interval)
    else:
        interval = datetime.timedelta(days=1)

    to_time = from_time = None
    if to:
        to_time = datetime.datetime.fromisoformat(to).astimezone()
    if from_:
        from_time = datetime.datetime.fromisoformat(from_).astimezone()

    if not to_time:
        if from_time and given_interval:
            to_time = from_time + interval
        else:
            to_time = datetime.datetime.now(datetime.timezone.utc)

    if not from_time:
        from_time = to_time - interval

    return from_time, to_time


interval_pattern = re.compile(
    r'((?P<days>\d+)d)?((?P<hours>\d+)h)?((?P<minutes>\d+)m)?((?P<seconds>\d+)s)?')


def parse_interval(interval: str):
    m = interval_pattern.fullmatch(interval)
    if not m:
        raise ValueError("Invalid interval")
    return datetime.timedelta(
        **dict(map(lambda x: (x[0], int(x[1])),
                   filter(lambda x: x[1] is not None,
                            m.groupdict().items()))))


def main():
    opts = docopt.docopt(__doc__)
    if opts['--verbose']:
        level = logging.DEBUG
    else:
        level = logging.INFO

    logging.basicConfig(level=level,
                        handlers=[RichHandler(rich_tracebacks=True)])

    client = boto3.client('logs')
    if opts['list-streams']:
        streams = get_streams(client, opts['LOG_GROUP'],
                              opts['STREAM_PREFIX'])
        print("Available log streams:\n\n  " + "\n  ".join(streams))
    else:
        with rp.Progress(
                rp.SpinnerColumn(),
                rp.TextColumn("[progress.description]{task.description}"),
                rp.BarColumn(),
                rp.TextColumn(
                    "[progress.percentage]{task.percentage:>3.0f}%"),
                rp.TimeRemainingColumn(),
                rp.TimeElapsedColumn()) as progress:

            from_time, to_time = parse_period(opts['--from'],
                                              opts['--interval'],
                                              opts['--to'])

            log_download(client,
                         opts['LOG_GROUP'], opts['STREAM_PREFIX'],
                         opts['OUTPUT'],
                         from_time=from_time, to_time=to_time,
                         progress=progress)


if __name__ == "__main__":
    main()



"""
Usage: log-generate [options] FILE OUTPUT_LOG_FILE

Options:
  --help            This help.

"""
import docopt
import datetime
import random


def main():
    opts = docopt.docopt(__doc__)

    now = datetime.datetime.now(tz=datetime.timezone.utc)

    def seconds_gen():
        total = 0
        while True:
            yield total
            total += random.randint(1, 100)

    seconds = seconds_gen()

    with open(opts['OUTPUT_LOG_FILE'], "w") as outfile:
        for secs, ll in reversed(list(zip(seconds, reversed(open(opts['FILE'], "r").readlines())))):
            print(f"{(now - datetime.timedelta(seconds=secs)).isoformat()} {ll.rstrip()}", file=outfile)


if __name__ == "__main__":
    main()


import setuptools
from pathlib import Path

here = Path(__file__).parent.resolve()

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open(here / "src" / "metsuri" / "VERSION", "r") as version_file:
    version = version_file.read().strip()


setuptools.setup(
    name="metsuri",
    version=version,
    author="Sami J. Lehtinen",
    author_email="sami.lehtinen@kone.com",
    description="Serial log collector and uploader",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/koneopen/metsuri",
    packages=setuptools.find_packages(where="src"),
    package_dir={"": "src"},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=["docopt>=0.6.2", "boto3>=1.16", "pyserial>=3.4",
                      "rich>=9.0"],
    extras_require={
        'dev': ["freezegun", "pytest", "tox", "tox-pyenv", "wheel"]
    },
    entry_points={
        'console_scripts': ["log-download=metsuri.log_download:main",
                            "log-uploader=metsuri.log_uploader:main",
                            "serial-logger=metsuri.serial_logger:main",
                            "log-check=metsuri.log_check:main",
                            "log-generate=metsuri.log_generate:main"]
    },
    package_data={
        "metsuri": ["VERSION"]
    }

)